/*
 * $Id: kebook.h,v 1.5 2001/01/31 07:13:19 hige Exp $
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#ifndef KEBOOK_H 
#define KEBOOK_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <qdir.h>
#include <kmainwindow.h>
#include <kprogress.h>

#include "kebookapp.h"
#include "constants.h"

class KEBookWidget;

class KEBookProgressEvent : public QEvent
{
public:
    KEBookProgressEvent(Type type, int index, int count, int percent)
        : QEvent(type),
          m_index(index),
          m_count(count),
          m_percent(percent) {}
    int  getPercent() {return m_percent;}
    void setPercent(int percent) {m_percent = percent;}

    int  getIndex() {return m_index;}
    void setIndex(int index) {m_index = index;}

    int getCount() {return m_count;}
    void setCount(int count) {m_count = count;}
    
private:
    int m_index;
    int m_count;
    int m_percent;
};

/**
 * This class serves as the main window for KEBook.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 * @version 0.1
 */
class KEBook
    : public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    KEBook();
    /**
     * Copy Constructor (== empty)
     */ 
    KEBook(KEBook&);

    /**
     * Default Destructor
     */
    virtual ~KEBook();

    /**
     * Event Reciever
     */
    virtual bool event(QEvent* ev);

    void preLookupWord(const char* word);
    void postLookupWord();
    
    void setSubbook(int i);
    void setSubbook(const KEBookApp::DictSpec& dictspec);

signals:
    void        sigSelectedSubbook();
    void        sigReleaseSubbook();
    void        sigSuspendGUIParts();
    void        sigResumeGUIParts();
    void        sigAddMatchWordsList(int index, int count);
    void        sigCloseMainWindow();
    
public slots:

private slots:
    /**
     * These functions are called to open dialog for setting.
     */
    void openDictPathDlg(const char* dict_path);
    void openDictPathDlg()      {openDictPathDlg(getApp()->getDictPath());}

    void openSelectSubbookDlg();
    void openSelectWayToSearchDlg();
    //void openEditAccelDlg()     {view->openEditAccelDlg();}
    
    /**
     * These functions are called to read or write config.
     */
    void readConfig() {readProperties(getApp()->config());}
    void saveConfig() {saveProperties(getApp()->config());}

    /**
     * This function is called to show the dictionary menu.
     */
    void showDictMenu();
    
    /**
     * This function is called to show the dictionary copyright.
     */
    void showDictCopyright();

    /**
     * This function is called on release subbook
     */
    void onReleaseSubbook();

    /**
     * This function is called on set subbook 
     */
    void onSetSubbook();

    /**
     * This function is called on set the dictionary path containing
     * dictionary.
     */
    void onSetDictPath();
    

    /**
     * tell user's choice of search method to search engine
     * (Menu id of the search method will be set to paramter i.)
     */
    void tellWayToSearchToEngine(int menu_id);

    /**
     * On Selected dictionary menu
     */
    void onSelectDictionaryFromList(int menu_id);

    /**
     * On selected DictDirHistory menu
     */
    void onSelectDictDirFromList(int menu_index);

    /**
     * re-create menu of available dictionaries
     */
    bool rescanDictionariesInCDROM();

    /**
     * quit KEBook
     */
    void quitKEBook() {close();}
    
protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties(KConfig *);

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref savePropertiesx
     */
    void readProperties(KConfig *);

    virtual void closeEvent(QCloseEvent* ev);
    virtual void keyPressEvent(QKeyEvent* ev);
    virtual void resizeEvent(QResizeEvent* ev);
    virtual void moveEvent(QMoveEvent* ev);

private: // Internal
    /**
     * Create GUI Parts
     */
    // Create menus
    void createMainMenu();
    void createFileMenu();
    void createEditMenu();
    void createShowMenu();
    void createGoMenu();
    void createOptionMenu();
    void createCDROMMenu();
    void createHelpMenu();
    
    void createToolBar();
    void createWayToSearchPopupMenu();
    
    void createStatusBar();

    void updateDictDirHistoryMenu();
    
    /**
     * This function shows error message when opening dictionary fails.
     */
    void showErrorOnOpeningDictionary(const char* hint);
    
    /**
     * This function shows message on the result view prompting to set
     * directory path to user's CD-ROM book.
     */
    void promptToSelectDirectory();

    /**
     * This function shows message on the result view prompting to select
     * one subbook from available titles in user's CD-ROM book.
     */
    void promptToSelectDictionary();

    /**
     * This function updates current status of search method selection
     * displayed on popup menu and toolbar
     */
    void updateSelectionOfWayToSearch();

    /**
     *
     */
    bool setDictPath(const QString& path);

    /**
     * About properties 
     */
    void saveDictDirHistoryProperties(KConfig* config);
    void readSubbookInfoListProperties(KConfig* config);
    void readDictDirHistoryProperties(KConfig* config);
    
private:
    KEBookWidget*   view;

    /**
     * menu of available dictionaries at runtime
     */
    QPopupMenu*     m_dictionary_list_popup;
    
    /**
     * menu of search method
     */
    QPopupMenu*     m_way_to_search_popup;

    /**
     * menu of path of dictionary     
     */
    QPopupMenu*     m_dictdir_popup;
    
    /**
     * GUI parts in the status bar
     */
    KProgress*      m_pb_lookingup;
    QLabel*         m_lbl_message;

    /**
     * For choose way to search
     */
    friend class WayToSearchTableElement;
    vector<WayToSearchTableElement>* m_way_to_search_table;

    /**
     * Accelerator key
     */
    QAccel* m_function_key;

    /**
     * Current Geometry
     */
    QRect  m_curgeometry;
};

class WayToSearchTableElement : public QObject {
   Q_OBJECT
    
public:
    WayToSearchTableElement() {}
    WayToSearchTableElement(KEBook* parent,
                            KEBookApp::WAY_TO_SEARCH way, int menu_id,
                            const QString& description, const char* action, int accel,
                            QString iconfile)
        : m_parent(parent),
          m_way(way), m_menu_id(menu_id),
          m_description(description), m_action(action),
          m_accel(accel),
          m_iconfile(iconfile) {}
    WayToSearchTableElement(const WayToSearchTableElement& another)
        : QObject(),
          m_parent(another.m_parent),
          m_way(another.m_way), m_menu_id(another.m_menu_id),
          m_description(another.m_description), m_action(another.m_action),
          m_accel(another.m_accel),
          m_iconfile(another.m_iconfile) {}
    WayToSearchTableElement& operator=(const WayToSearchTableElement& another){
        m_parent = another.m_parent;
        m_way = another.m_way;
        m_menu_id = another.m_menu_id;
        m_description = another.m_description;
        m_action = another.m_action;
        m_accel = another.m_accel;
        m_iconfile = another.m_iconfile;
        return *this;
    }

    KEBookApp::WAY_TO_SEARCH    getWayToSearch() {return m_way;}
    int                         getMenuId() {return m_menu_id;}
    const QString&              getDescription() {return m_description;}
    const char*                 getAction() {return m_action;}
    int                         getAccel() {return m_accel;}
    QString                     getIconFile() {return m_iconfile;}
    
public slots:
    void                        setWayToSearch()  {m_parent->tellWayToSearchToEngine(m_menu_id);}
     
private:
    KEBook*                     m_parent;
    KEBookApp::WAY_TO_SEARCH    m_way;
    int                         m_menu_id;
    QString                     m_description;
    const char*                 m_action;
    int                         m_accel;
    QString                     m_iconfile;
};
#endif // KEBOOK_H 
