/*
 * $Id: kebookapp.h,v 1.2 2001/01/13 19:46:35 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef KEBOOKAPP_H
#define KEBOOKAPP_H

#include <kapp.h>
#include <qdir.h>
#include <stl.h>

#include "cebook.hpp"
#include "thread.hpp"
#include "constants.h"

class KAccel;

class KEBookApp : public KApplication
      , public ebook::CEBook,
        public ebook::CEBook::LookupClient,
        public thread::Runnable
{
    Q_OBJECT

public:
    KEBookApp();
    ~KEBookApp();
    
signals:
    void                    sigReleaseSubbook();
    void                    sigSetSubbook();
    void                    sigSetDictPath();
    
public:
    // Override CEBook::LookupClient method
    void                    showProgress(int index, int count, int percent);
    void                    enterCriticalSection(void* obj) {
        m_lookup_thread->enterCriticalSection(obj);
    }
    void                    leaveCriticalSection(void* obj) {
        m_lookup_thread->leaveCriticalSection(obj);
    }
    bool                    isStopped() {return m_stopped_lookingup;}
    
    /**
     * These methods is called for operate Dictionary.
     */
    void initDictionary();
    void getSubbookTitleList(vector<string>& list);

    void setSubbook(SubbookInfoListIterator it);
    void setSubbookWithIndex(int i) {setSubbook(getBeginOfSubbookInfoList()+i);}
    void setDictPath(const char* dict);
    void lookupWord(const char* word);
    void lookupWord(const string& word) {lookupWord(word.c_str());}
    /**
     * These methods operate result history
     */
    void pushResult(const string& word, const string& content);
    bool getCurrentResult(string& title, string& content);
    bool movePrev(string& title, string& content);
    bool moveNext(string& title, string& content);
    bool isHistoryBegin(){
        return (m_cur_result_pos == m_result_history.begin()) ? true : false;
    }
    bool isHistoryEnd() {
        ResultHistory::iterator i = m_result_history.end();
        return (m_cur_result_pos == (--i)) ? true : false;
    }
    
    const string&               getLookupResult(int i) const {
        return (*(m_cur_result->begin()+i)).second;
    }
    const string&               getMatchWord(int i) const {
        return (*(m_cur_result->begin()+i)).first;
    }
    LookupResult::const_iterator getBeginOfMatchWords() const {
        return m_cur_result->begin();
    }
    LookupResult::const_iterator getEndOfMatchWords() const {
        return m_cur_result->end();
    }
    LookupResult::size_type getMatchWordsCount() const {
        return m_cur_result->size();
    }
    const string&   getEBErrorMessage() const   {return m_cur_error_msg;}
    bool            isLookingupError()          {return m_iserror;}
    
    void            clearCurResult()            {m_cur_result->clear();}

    WAY_TO_SEARCH      getWayToSearch()                      {return m_way_to_search;}
    void            setWayToSearch(WAY_TO_SEARCH way_to_search) {m_way_to_search = way_to_search;}
    
    /**
     * dictionary specification (combination of directory path, subbook title)
     */
    struct DictSpec {
        string directory;
        string title;
        int    code;
    };
    typedef vector<DictSpec>                    DictSpecList;
    typedef vector<DictSpec>::const_iterator    DictSpecListIterator;
    typedef deque<string>                      DictDirHistory;
    typedef deque<string>::const_iterator      DictDirHistoryIterator;

    /**
     * obtain list of available dictionaries and store them in the memory
     */
    bool rescanDictSpecListInCDROM();
    
    /**
     * return the begin/end iterator of dictionaries
     */
     DictSpecListIterator getBeginOfDictSpecListInCDROM() const
         {return m_dictionary_spec_list.begin();}
     DictSpecListIterator getEndOfDictSpecListInCDROM() const
         {return m_dictionary_spec_list.end();}

    /**
     * open dictionary specified by DictSpec value
     */
    void setDictionary(const DictSpec& dictspec);
    
public slots:
    /**
     * This function is called to stop looking up the word.
     */
    void stopLookingUp() {m_stopped_lookingup = true;}

    /**
     * This slot quit KEBook.
     */
    void quit();

public:
    /**
     * Thread method
     */
    void*   run(void* args);
    void    start(void* args);
    bool    isLookingupThreadAlive()
        {return (m_lookup_thread) ? m_lookup_thread->isAlive(): false;}

public:
    /**
     * Operate history of directory containing dictionary
     */
    DictDirHistory::const_iterator getBeginOfDictDirHistory() {return m_dictdir_history.begin();}
    DictDirHistory::const_iterator getEndOfDictDirHistory() {return m_dictdir_history.end();}
    
    void  appendDictDirHistory(const char* path);

private:
    /**
     * To treat a temporary directory for font files.
     */
    bool makeTempRouteDir();
    bool makeTempFontsDir(string& receieve_fontsdir);
    bool deleteTempDir();
    
private:
    typedef list< pair<string, string> >    ResultHistory;
    static string                           RESULTHISTORY_WORD(ResultHistory::iterator it) {return (*it).first;}
    static string                           RESULTHISTORY_CONTENT(ResultHistory::iterator it) {return (*it).second;}
    int                                     m_max_history_size;
    ResultHistory                           m_result_history;
    ResultHistory::iterator                 m_cur_result_pos;
    LookupResult*                           m_cur_result;
    string                                  m_cur_error_msg;
    bool                                    m_stopped_lookingup;
    thread::Thread*                         m_lookup_thread;
    WAY_TO_SEARCH                           m_way_to_search;
    QDir                                    m_tempdir;
    bool                                    m_iserror;
    DictSpecList                            m_dictionary_spec_list;
    DictDirHistory                          m_dictdir_history;
    DictDirHistory::size_type               m_dictdir_history_maxsize;
};
inline KEBookApp* getApp() {return (KEBookApp*)kapp;}

#endif // KEBOOKAPP_H
