/*
 * $Id: thread.cpp,v 1.1.1.1 2000/05/25 06:09:58 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <signal.h>

#include  "thread.hpp"

using namespace thread;

#define EMPTY 0

int
Thread::m_instance_num = 0;

#ifdef HAVE_PTHREAD
pthread_mutex_t
Thread::m_mutex = PTHREAD_MUTEX_INITIALIZER;

Thread::MutexArray
Thread::g_mutexes;
#endif

void Thread::initAMutex()
{
#ifdef HAVE_PTHREAD
    if (m_instance_num == 0) {
        int rc;
        if ((rc = pthread_mutex_init(&m_mutex, EMPTY)) != 0) {
            throw ThreadException(rc);
        }
    }
#endif
}
Thread::Thread()
    : m_target(EMPTY),
      m_name("No name"),
      m_alive(false)
{
    initAMutex();
}
Thread::Thread(const char* name)
    : m_target(this),
      m_name(name),
      m_alive(false)
{
    initAMutex();
    m_instance_num++;
}
Thread::Thread(Runnable* target, const char* name)
    : m_target(target),
      m_name(name),
      m_alive(false)
{
    initAMutex();
    m_instance_num++;
}
Thread::Thread(const Thread& th)
    : Runnable(),
      m_target(th.m_target),
      m_name(th.m_name),
      m_alive(false)
{
    initAMutex();
    m_instance_num++;
}

Thread::~Thread()
{
#ifdef HAVE_PTHREAD    
    if (m_alive)
        kill();
    destroyMutexes();
    if (--m_instance_num == 0) {
        MutexArray::iterator i;
        i = g_mutexes.begin();
        while (i != g_mutexes.end()) {
            pthread_mutex_destroy(&(*i++).second.mutex);
        }
        pthread_mutex_destroy(&m_mutex);
    }
#else
    m_instance_num--;
#endif
}
void* Thread::thread_runner(ThreadRunnerArgs* args)
{
    void* rvalue = args->th->run(args->arg);
    *args->alive = false;
    unlockMutexes();
    return rvalue;
}
void* Thread::run(void* arg)
{
    void* return_value = m_target->run(arg);
    return return_value;
}
typedef void * (*THREAD_FUNC)(void *);
void Thread::start(void* arg)
{
    m_alive = true;
    m_args.th = m_target;
    m_args.arg = arg;
    m_args.alive = &m_alive;
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_create(&m_thread_id, EMPTY,
                             (THREAD_FUNC)thread_runner, &m_args)) != 0) {
        m_alive = false;
        throw ThreadException(rc);
    }
#else
    thread_runner(&m_args);
#endif
}
// Stop/Term/Kill thread
void Thread::stop()
{
    if (!m_alive) {
        return;
    }
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_kill(m_thread_id, SIGQUIT)) != 0
        && (rc = pthread_detach(m_thread_id)) != 0) {
        throw ThreadException(rc);
    }
    if ((rc = pthread_detach(m_thread_id)) != 0) {
        throw ThreadException();
    }
    m_alive = false;
#endif
}
void Thread::term()
{
    if (!m_alive) {
        return;
    }
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_kill(m_thread_id, SIGTERM)) != 0) {
        throw ThreadException(rc);
    }
    if ((rc = pthread_detach(m_thread_id)) != 0) {
        throw ThreadException(rc);
    }
    m_alive = false;
#endif
}
void Thread::kill()
{
    if (!m_alive) {
        return;
    }
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_kill(m_thread_id, SIGKILL)) != 0) {
        throw ThreadException(rc);
    }
    if ((rc = pthread_detach(m_thread_id)) != 0) {
        throw ThreadException(rc);
    }
    m_alive = false;
#endif
}

void Thread::enterCriticalSection()
{
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_mutex_lock(&m_mutex)) != 0) {
        throw ThreadException(rc);
    }
#endif
}
void Thread::leaveCriticalSection()
{
#ifdef HAVE_PTHREAD
    int rc;
    if ((rc = pthread_mutex_unlock(&m_mutex)) != 0) {
        throw ThreadException(rc);
    }
#endif
}
void Thread::enterCriticalSection(void* obj)
{
#ifdef HAVE_PTHREAD
    int rc;
    pthread_mutex_t mutex;
    if (g_mutexes.find(obj) == g_mutexes.end()) {
        pthread_mutex_init(&mutex, EMPTY);
        MUTEX_INFO mutex_info;
        mutex_info.thread_id = pthread_self();
        mutex_info.mutex = mutex;
        g_mutexes[obj] = mutex_info;
    } else {
        mutex = g_mutexes[obj].mutex;
    }
    if ((rc = pthread_mutex_lock(&mutex)) != 0) {
        throw ThreadException(rc);
    }    
#endif
}
void Thread::leaveCriticalSection(void* obj)
{
#ifdef HAVE_PTHREAD
    if (g_mutexes.find(obj) == g_mutexes.end()) {
        return;
    }
    pthread_mutex_t mutex = g_mutexes[obj].mutex;
    int rc;
    if ((rc = pthread_mutex_unlock(&mutex)) != 0) {
        throw ThreadException(rc);
    }
#endif
}
void Thread::unlockMutexes()
{
#ifdef HAVE_PTHREAD
    MutexArray::iterator i = g_mutexes.begin();
    while (i != g_mutexes.end()) {
        if (pthread_equal((*i).second.thread_id,
                          pthread_self())) {
            pthread_mutex_unlock(&(*i).second.mutex);
        }
        i++;
    }
#endif
}
void Thread::destroyMutexes()
{
#ifdef HAVE_PTHREAD
    MutexArray::iterator i = g_mutexes.begin();
    int rc;
    while (i != g_mutexes.end()) {
        if ((rc = pthread_mutex_destroy(&(*i).second.mutex)) != 0) {
            i++;
        } else {
            g_mutexes.erase(i);
            i = g_mutexes.begin();
        }
    }
#endif
}
void* Thread::join()
{
#ifdef HAVE_PTHREAD
    void* p;
    int rc;
    if ((rc = pthread_join(m_thread_id, &p)) != 0) {
        throw ThreadException(rc);
    }
    return p;
#else
    return EMPTY;
#endif
}

bool Thread::operator==(const Thread& another) const
{
#ifdef HAVE_PTHREAD
    if (pthread_equal(this->m_thread_id, another.m_thread_id))
        return true;
    else
        return false;
#else
    return this == &another;
#endif    
}
